unit  GAB_Alg;               { Gain Adaptive Back Propagation Algorithm }
                             { author Petr Bozovsky }
interface
uses  BackProp;

type

GAB = ^TGAB;
TGAB = object (TBP)
      Lambda, LambdaOld : array [Layers,Neurons] of real;
      Ksi     : array [Layers,Neurons] of real;
                        { Ksi[i,j] ... potential of neuron j in layer i }
      eta2, alpha2 : real;
   constructor Init;
   procedure State;                          virtual;
   procedure ChangeParams ( Layer:Layers );  virtual;
   procedure InitNetwork;                    virtual;
private
   function S ( Ks,lambd:real ) : real;
end;


implementation

constructor TGAB.Init;
begin TBP.Init;
end;

function TGAB.S ( Ks,lambd:real ) : real;        { neuron transfer function }
const RB     = 30;         { where to extrapolate the sigmoid by a constant }
var   inp : real;
begin inp:=lambd*Ks;
      if inp>30 then S:=1
                else if inp<-30 then S:=0
                                else S:=1/(1+exp(-inp));
end;

procedure TGAB.State;                            { new state of the network }
var   Layer : Layers;
      j     : NeurThrs;
      k     : Neurons;
      KsiAux: real;                                      { neuron potential }
begin for j:=1 to NoofN[0] do
        x[0,j]:=Input[j];                         { set bottom layer inputs }
      for Layer:=1 to NoofL do
        for k:=1 to NoofN[Layer] do
          begin KsiAux:=0;
                for j:=0 to NoofN[Layer-1] do
                  KsiAux:=KsiAux+w[Layer-1,j,k]*x[Layer-1,j];
                Ksi[Layer,k]:=KsiAux;
                x[Layer,k]:=S(KsiAux,Lambda[Layer,k])       { neuron output }
          end
end;                        { x[NoofL,k] is an actual output of the network }

procedure TGAB.ChangeParams ( Layer:Layers );      { new weights and slopes }
var   j     : NeurThrs;
      k     : Neurons;
      DeltaAux,LambdaAux : real;            { auxiliary temporal variables }
begin Inc(Layer);
      for k:=1 to NoofN[Layer] do
        begin DeltaAux:=Delta[Layer,k];
              LambdaAux:=Lambda[Layer,k];
              Lambda[Layer,k]:=LambdaAux-eta2*DeltaAux*Ksi[Layer,k]+
                               alpha2*(LambdaAux-LambdaOld[Layer,k]);
              LambdaOld[Layer,k]:=LambdaAux;
              Delta[Layer,k]:=DeltaAux*LambdaAux;
        end;
      Dec(Layer);
      TBP.ChangeParams(Layer);
end;

procedure TGAB.InitNetwork;  { !! network parameters initialization routine }
var   i : Layers;            { this is the only task dependent procedure !! }
      k : Neurons;
begin TBP.InitNetwork;
      for i:=1 to NoofL do
        for k:=1 to NoofN[i] do
         Lambda[i,k]:=6*(Random-0.5)/10;
      LambdaOld:=Lambda;
      eta2:=0.3;  alpha2:=0.7;
end;

{ begin - no initialization part }
  end.